use std::convert::TryFrom;

use lazy_static::lazy_static;
use log::trace;
use regex::Regex;
use ros_message::{MessagePath, Msg};

use crate::{index::ConnInfo, Result};

fn parse_msg(message_name: &str, msgdef: &str) -> Result<Msg> {
    trace!("message definition: {}", msgdef);
    let path = MessagePath::try_from(message_name)?;
    let msgtype = Msg::new(path, msgdef)?;
    Ok(msgtype)
}

lazy_static! {
    static ref BOUNDARY_RE: Regex = Regex::new(r"\r?\n==+\r?\nMSG: ([^\r\n]+)\r?\n").unwrap();
}

pub fn parse_msgs(conn: &ConnInfo) -> Result<Vec<Msg>> {
    let msgdefs = &conn.message_definition;
    let mut name = conn.datatype.clone();
    let mut begin = 0usize;
    let mut msgs = Vec::new();

    for cap in BOUNDARY_RE.captures_iter(msgdefs) {
        let boundary_range = cap.get(0).unwrap();
        let end = boundary_range.start();

        let msgdef = &msgdefs[begin..end];
        let msgtype = parse_msg(&name, msgdef)?;
        msgs.push(msgtype);

        name = cap[1].to_string();
        begin = boundary_range.end();
    }

    let msgdef = &msgdefs[begin..];

    let msg = parse_msg(&name, msgdef)?;
    msgs.push(msg);

    Ok(msgs)
}
